/* SRCDrop - Filename: AUDIO.H
 *
 *   Copyright (c) 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Portions Copyright 2000-2002, Michael Smith <msmith@labyrinth.net.au>
 */


#ifndef AUDIO_H_INCLUDED
#define AUDIO_H_INCLUDED

#include "resample.h"
#include <stdio.h>

#define MAXWAVESIZE     4294967040LU

enum {
	WAV_NO_FMT = 0,
	WAV_FMT_8BIT,
	WAV_FMT_16BIT,
	WAV_FMT_24BIT,
	WAV_FMT_32BIT,
	WAV_FMT_FLOAT,
} file_formats;

#ifndef	WAVE_FORMAT_PCM
#	define	WAVE_FORMAT_PCM                0x0001
#endif
#ifndef	WAVE_FORMAT_IEEE_FLOAT
#	define	WAVE_FORMAT_IEEE_FLOAT         0x0003
#endif
#ifndef	WAVE_FORMAT_EXTENSIBLE
#	define	WAVE_FORMAT_EXTENSIBLE         0xfffe
#endif

enum {
	UNDEFINED = 0,
	DEFINED
} channel_config;

typedef struct
{
	int  (*id_func)(unsigned char *buf, int len); /* Returns true if can load file */
	int  id_data_len; /* Amount of data needed to id whether this can load the file */
	int  (*open_func)(FILE *in, src_proc_opt *opt, unsigned char *oldbuf, int buflen);
	void (*close_func)(void *);
	char *format;
	char *description;
} input_format;


typedef struct {
	unsigned short format;
	unsigned short channels;
	unsigned int   channel_mask;
	unsigned int   samplerate;
	unsigned int   bytespersec;
	unsigned short align;
	unsigned short samplesize;
} wav_fmt;

typedef struct {
	unsigned short channels;
	unsigned int   channel_map;
	short          samplesize;
	unsigned long  totalsamples;
	unsigned long  samplesread;
	FILE           *f;
	short          bigendian;
} wavfile;

typedef struct {
	unsigned short channels;
	unsigned int   totalframes;
	short          samplesize;
	int            rate;
	int            offset;
	int            blocksize;
} aiff_fmt;

typedef wavfile aifffile; /* They're the same */

input_format *open_audio_file(FILE *in, src_proc_opt *opt);

int wav_open(FILE *in, src_proc_opt *opt, unsigned char *oldbuf, int buflen);
int wav_id(unsigned char *buf, int len);
void wav_close(void *);

long wav_read(void *, float *buffer, int samples);
long wav_ieee_read(void *, float *buffer, int samples);

typedef struct
{
	int           outputFormat;
	FILE          *sndfile;
	unsigned long samplerate;
	unsigned int  bits_per_sample;
	unsigned int  channels;
	unsigned long samples;
} output_audio_file;

output_audio_file *open_output_audio_file(src_proc_opt *opt);
int write_audio_file(output_audio_file *aufile, void *sample_buffer, int samples);
void close_audio_file(output_audio_file *aufile, src_proc_opt *opt);
static int write_wav_header(output_audio_file *aufile, src_proc_opt *opt);
static int write_wav_extensible_header(output_audio_file *aufile, src_proc_opt *opt);
static int write_audio_8bit(output_audio_file *aufile, void *sample_buffer, unsigned int samples);
static int write_audio_16bit(output_audio_file *aufile, void *sample_buffer, unsigned int samples);
static int write_audio_24bit(output_audio_file *aufile, void *sample_buffer, unsigned int samples);
static int write_audio_32bit(output_audio_file *aufile, void *sample_buffer, unsigned int samples);
static int write_audio_float(output_audio_file *aufile, void *sample_buffer, unsigned int samples);
void* output_to_PCM(float *input, void *samplebuffer, int channels, int samples, int format);

#endif /* AUDIO_H_INCLUDED */
