/* SRCDrop - Filename: ENCTHREAD.C
 *
 *   Copyright (c) 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <time.h>
#include <string.h>
#include <locale.h>

#include "misc.h"
#include "mscver.h"
#include "audio.h"
#include "resample.h"
#include "i18n.h"

extern char   config_outdir[MAX_PATH];
extern int    encoding_done,
	      encoding,
              animate,
              numfiles,
              totalfiles;
extern double file_complete;
extern char   fileName[];

typedef struct enclist_tag
{
	char *filename;
	struct enclist_tag *next;
} enclist_t;

enclist_t *head = NULL;

src_options opt = {NULL, 48000, 1, 0};

CRITICAL_SECTION mutex;

long encode_thread(LPVOID arg);

void encthread_init(void)
{
	DWORD thread_id;
	HANDLE thand;

	numfiles = 0;
	totalfiles = 0;
	file_complete = 0.0;

	InitializeCriticalSection(&mutex);

	thand = (HANDLE)CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)encode_thread, NULL, 0, &thread_id);
	if (thand == NULL) {
		// something bad happened, might want to deal with that, maybe...
	}
}

void encthread_addfile(char *file)
{
	char *filename;
	enclist_t *entry, *node;

	if (file == NULL)
		return;

	// create entry
	filename = strdup(file);
	entry = (enclist_t *)calloc(1, sizeof(enclist_t));

	entry->filename = filename;
	entry->next = NULL;

	EnterCriticalSection(&mutex);

	// insert entry
	if (head == NULL)
		head = entry;
	else {
		node = head;
		while (node->next != NULL)
			node = node->next;

		node->next = entry;
	}
	numfiles++;
	totalfiles++;

	LeaveCriticalSection(&mutex);
}

// the caller is responsible for deleting the pointer
char *_getfile()
{
	char *filename;
	enclist_t *entry;

	EnterCriticalSection(&mutex);

	if (head == NULL) {
		LeaveCriticalSection(&mutex);
		return NULL;
	}

	// pop entry
	entry = head;
	head = head->next;

	filename = entry->filename;
	free(entry);

	LeaveCriticalSection(&mutex);

	return filename;
}

void _nothing_prog(long total, long done)
{
	// do nothing
}

void _error(char *errormessage)
{
	// do nothing
}

void _update(long total, long done)
{
	file_complete = (double)done / (double)total;
}

void encoding_loop(void)
{
		char	*in_file;

		while (in_file = _getfile()) {
			char		*p,
					in_path[MAX_PATH],
					title[MAX_PATH],
					lpstrFilename[MAX_PATH],
					date_buf[12],
					time_buf[12];
			src_proc_opt	src_opts;
			char		*out_fn = NULL;
			FILE		*in,
					*out = NULL;
			int		foundformat = 0,
					j = 0,
					i,
					channels;
			long		rate;
			input_format	*format;

			animate = 1;

			set_filename(in_file);

			in = fopen(in_file, "rb");

			if (in == NULL) {
				error_handler("ERROR: Cannot open input file %s", in_file);
				numfiles--;
				continue;
			}

			/* Now, we need to select an input audio format */

			format = open_audio_file(in, &src_opts);
			if (format)
				foundformat=1;
			else {
				fclose(in);
				/* error reported by reader */
				numfiles--;
				continue;
			}

			/* Ok. We can read the file - so now open the output file */

			if (out == NULL) {
				/* Create a filename from existing filename, replacing extension with .samplerate.wav */
				int len,
				    len_ip;

				strcpy(in_path, in_file);
				len = strlen(in_path);
				while(in_path[len] != '\\')
					len--;
				if (in_path[len] == '\\') {
					len_ip = len + 1;
					for(i = 0; len < strlen(in_path); i++, len++)
						title[i] = in_path[len+1];
				}
				in_path[len_ip] = '\0';

				len = strlen(title);
				title[len] = '\0';
				while(title[len] != '.')
					len--;
				if (title[len] == '.')
					title[len] = '\0';

				len = strlen(config_outdir);
				if (config_outdir[len-1] == '\\')
					config_outdir[len-1] = '\0';
				p=config_outdir;

				if (!opt.file_suffix[0]) {
					if (!config_outdir[0])
						wsprintf(lpstrFilename,"%s%s__%d.wav",in_path, title, opt.resamplefreq);
					else if (p[0]=='\\')
						wsprintf(lpstrFilename,"%s%s__%d.wav",config_outdir,title, opt.resamplefreq);
					else
						wsprintf(lpstrFilename,"%s\\%s__%d.wav",config_outdir,title, opt.resamplefreq);
				}
				else {
					if (!config_outdir[0])
						wsprintf(lpstrFilename,"%s%s%s.wav",in_path, title, opt.file_suffix);
					else if (p[0]=='\\')
						wsprintf(lpstrFilename,"%s%s%s.wav",config_outdir,title, opt.file_suffix);
					else
						wsprintf(lpstrFilename,"%s\\%s%s.wav",config_outdir,title, opt.file_suffix);
				}

				out_fn = lpstrFilename;

				strcpy(fileName, out_fn);
			}

			/* Now, set the rest of the options */	
			src_opts.progress_update = _update;
			src_opts.filename = out_fn;
			channels = src_opts.channels;
			rate = src_opts.rate;
			src_opts.resamplefreq = opt.resamplefreq;
			src_opts.converter = opt.converter;
			src_opts.logerr = opt.logerr;

			if (opt.output_format)
				src_opts.format = opt.output_format;

			if (!src_opts.total_samples_per_channel)
				src_opts.progress_update = _nothing_prog;

			src_proc(&src_opts); /* Should we care about return val? */

			if (opt.logerr) {
				GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, NULL,
				              NULL, date_buf, 12);
				GetTimeFormat(LOCALE_USER_DEFAULT, LOCALE_NOUSEROVERRIDE,
				              NULL, NULL, time_buf, 12);
				error_handler("***********************************");
				error_handler(" %s - %s", date_buf, time_buf);
				error_handler(" -  Input filename: %s", in_file);
				error_handler(" - Output filename: %s", out_fn);
				error_handler(" No. of Channels: %d", channels);
				error_handler("   Sampling Rate: %dHz", rate);
				error_handler("   Total Samples: %d", src_opts.total_samples_per_channel);
				error_handler("          Length: %dseconds", src_opts.total_samples_per_channel/src_opts.rate);
				error_handler(" - Re-sampling to: %dHz", opt.resamplefreq);
			}

			format->close_func(src_opts.readdata);
			fclose(in);

			if (opt.delete_input) {
				if (remove(in_file) != 0) {
					error_handler("Error deleting input file '%s'\n", in_file);
					continue;
				}
			}

			numfiles--;
		} /* Finished this file, loop around to next... */
}

long encode_thread(LPVOID arg)
{
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	while (!encoding_done) {

		SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
		SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);

		if (encoding)
			encoding_loop();
		encoding = 0;
		file_complete = 0.0;
		animate = 0;
		totalfiles = 0;
		numfiles = 0;

		Sleep(500);
	} 

	DeleteCriticalSection(&mutex);
	return (0);
}

/******************************* end of encthread.c *******************************/

