/* SRCDrop - Filename: MAIN.C
 *
 *   Copyright (c) 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <shellapi.h>
#include <string.h>
#include <stdio.h>
#include <commctrl.h>
#include <shlobj.h>	// for Output Directory options

#include "resource.h"
#include "encthread.h"
#include "misc.h"
#include "mscver.h"
#include "resample.h"

#define LOSHORT(l)           ((SHORT)(l))
#define HISHORT(l)           ((SHORT)(((DWORD)(l) >> 16) & 0xFFFF))

#define INI_FILE "srcdrop.ini"

#define CREATEFONT(sz) \
	CreateFont((sz), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
		VARIABLE_PITCH | FF_SWISS, "")
   
char         fileName[MAX_PATH],
             CmdDir[MAX_PATH],
             config_outdir[MAX_PATH] = "";
int          width = 130,
             height = 130,
             frame = 0,
             encoding_done = 0,
             encoding = 0,
             totalfiles,
             numfiles,
             animate = 0;
double       file_complete;
HANDLE       event = NULL;
RECT         bar1,
             bar2,
             vbrBR;
HINSTANCE    hinst;
HBITMAP      hbm[7];
HMENU        menu;
HWND         g_hwnd;
HFONT        font2;

extern src_options opt;

SettingsSRC  iniSettings;     // iniSettings holds the parameters for the SRCdrop configuration.

static const char *resampleCaption[] =
{
	"192000", "176400", "128000", "96000", "88200", "64000", "48000", 
	"44100", "32000", "24000", "22050", "16000", "11025", "8000", 0
};

static const char *converter[] =
{
	"Best", "Medium", "Fast", 0
};

static const char *wavFormat[] =
{
	"Same as input", "Unsigned 8 bit PCM", "16 bit PCM", "24 bit PCM", "32 bit PCM", "32 bit Float", 0
};

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

BOOL CALLBACK ResampleProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;

/* 
 * Next function for Output Directory options
 */
static int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED) {
		SetWindowText(hwnd,"Select Output Directory");
		SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_outdir);
	}
	return 0;
}

/*
 * Write the .ini file using the current SRCdrop settings
 */
int WriteIniFile (const char* Filename)
{
	FILE* fp;

	if ((fp = fopen (Filename, "w")) == NULL)
		return EOF;                             // could not open file

	fprintf (fp, "[SRCDrop]\n");
	fprintf (fp, "Window_X=%i\n"          , iniSettings.window_x     );
	fprintf (fp, "Window_Y=%i\n"          , iniSettings.window_y     );
	fprintf (fp, "Always_on_top=%i\n"     , iniSettings.always_on_top);
	fprintf (fp, "Logerr=%i\n"            , iniSettings.logerr       );
	fprintf (fp, "DeleteInput=%i\n"       , iniSettings.delete_input );
	fprintf (fp, "ReSampleRate=%i\n"      , iniSettings.resample_rate);
	fprintf (fp, "Converter=%i\n"         , iniSettings.converter    );
	fprintf (fp, "WavFormat=%i\n"         , iniSettings.wav_format   );
	fprintf (fp, "FileSuffix=%s\n"        , iniSettings.file_suffix  );
	fprintf (fp, "OutputDirMode=%i\n"     , iniSettings.out_dir_mode );
	fprintf (fp, "OutputDir=%s\n"         , iniSettings.out_dir      );
	return fclose (fp);
}

/*
 * Read the .ini file and set the SRCdrop settings
 */
int ReadIniFile (FILE* fp)
{
	char  buff [256];
	int   val;

	rewind (fp);
	fgets (buff, sizeof buff, fp);

	if (0 != memcmp ( buff, "[SRCDrop]", 9))
		return EOF;

	while (fgets (buff, sizeof buff, fp) != NULL) {
		if      (1 == sscanf (buff, "Window_X=%d"         , &val))  iniSettings.window_x      = val;
		else if (1 == sscanf (buff, "Window_Y=%d"         , &val))  iniSettings.window_y      = val;
		else if (1 == sscanf (buff, "Always_on_top=%d"    , &val))  iniSettings.always_on_top = val;
		else if (1 == sscanf (buff, "Logerr=%d"           , &val))  iniSettings.logerr        = val;
		else if (1 == sscanf (buff, "DeleteInput=%d"      , &val))  iniSettings.delete_input  = val;
		else if (1 == sscanf (buff, "ReSampleRate=%d"     , &val))  iniSettings.resample_rate = val;
		else if (1 == sscanf (buff, "Converter=%d"        , &val))  iniSettings.converter     = val;
		else if (1 == sscanf (buff, "WavFormat=%d"        , &val))  iniSettings.wav_format    = val;
		else if (1 == sscanf (buff, "FileSuffix=%64[^\r\n]"     , iniSettings.file_suffix));
		else if (1 == sscanf (buff, "OutputDirMode=%d"    , &val))  iniSettings.out_dir_mode  = val;
		else if (1 == sscanf (buff, "OutputDir=%245[^\r\n]"     , iniSettings.out_dir));
	}

	return 0;
}

/*
 * Get SRCdrop settings at startup, writes .ini file, if not present
 */
void GetSRCSettings (void)
{
	FILE*  fp = NULL;
	char   PathAndName [MAX_PATH];

	strcpy(PathAndName, CmdDir);
	strcat(PathAndName, INI_FILE);

	// set default values
	iniSettings.window_x      =   64;        // default box position (x co-ord)
	iniSettings.window_y      =   64;        // default box position (y co-ord)
	iniSettings.always_on_top =    8;        // default = on
	iniSettings.logerr        =    0;        // default = off
	iniSettings.delete_input  =    0;        // default = off
	iniSettings.resample_rate =48000;        // default = 48000
	iniSettings.converter     =    1;        // default = Medium
	iniSettings.wav_format    =    0;        // default = Same as input
	strcpy(iniSettings.file_suffix, "");     // default = blank
	iniSettings.out_dir_mode  =    0;        // default = same as input dir
	strcpy(iniSettings.out_dir, "");         // default = blank

	// Read INI_FILE
	if ((fp = fopen (PathAndName, "r")) == NULL) {    // file does not exist: write it!
		WriteIniFile(PathAndName);
	}
	else {                                              // file does exist: read it!
		ReadIniFile (fp);
		fclose (fp);
	}

	return;
}

void set_out_dir_mode(int v) {
	iniSettings.out_dir_mode = v;
}

void set_out_dir(char v[MAX_PATH]) {
	strcpy(iniSettings.out_dir, v);
}

void set_always_on_top(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_ONTOP, v ? MF_CHECKED : MF_UNCHECKED);
	SetWindowPos(hwnd, v ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW | SWP_NOMOVE);
	iniSettings.always_on_top = v;
}

void set_logerr(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_LOGERR, v ? MF_CHECKED : MF_UNCHECKED);
	iniSettings.logerr = v;
	set_use_dialogs(v);
	opt.logerr = v;
}

void set_delInput(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_DELINPUT, v ? MF_CHECKED : MF_UNCHECKED);
	opt.delete_input = v;
	iniSettings.delete_input = v;
}

void set_resample_rate(int v)
{
	opt.resamplefreq = v;
	iniSettings.resample_rate = v;
}

void set_converter(int v)
{
	opt.converter = v;
	iniSettings.converter = v;
}

void set_wav_format(int v)
{
	opt.output_format = v;
	iniSettings.wav_format = v;
}

void set_file_suffix(char *v)
{
	opt.file_suffix = strdup(v);
	strcpy(iniSettings.file_suffix, v);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	static char szAppName[] = "SRCdropWin";
	char        *p;
	HWND        hwnd;
	MSG         msg;
	WNDCLASSEX  wndclass;
	const int   width = 130,
	            height = 130;
	int         x,
	            y;

	hinst = hInstance;

	wndclass.cbSize        = sizeof(WNDCLASSEX);
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = WndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInstance;
	wndclass.hIcon         = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szAppName;
	wndclass.hIconSm       = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON2));

	RegisterClassEx(&wndclass);

	GetModuleFileName(NULL, CmdDir, MAX_PATH);
	p = strrchr(CmdDir, '\\') + 1;
	p[0] = '\0';

	GetSRCSettings();

	x = max(min(iniSettings.window_x, GetSystemMetrics(SM_CXSCREEN) - width), 0);
	y = max(min(iniSettings.window_y, GetSystemMetrics(SM_CYSCREEN) - height), 0);

	hwnd = CreateWindow(szAppName, "SRCdrop", WS_POPUP | WS_DLGFRAME, x, y,
		width, height, NULL, NULL, hInstance, NULL);

	g_hwnd = hwnd;

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	font2 = CREATEFONT(10);

	SetTimer(hwnd, 1, 80, NULL);

	/*
	 * Apply all the .ini settings
	 */

	set_always_on_top(hwnd, iniSettings.always_on_top);
	set_logerr(hwnd, iniSettings.logerr);
	set_delInput(hwnd, iniSettings.delete_input);
	set_resample_rate(iniSettings.resample_rate);
	set_converter(iniSettings.converter);
	set_wav_format(iniSettings.wav_format);
	set_file_suffix(iniSettings.file_suffix);
	if(iniSettings.out_dir_mode) {
		set_out_dir(iniSettings.out_dir);
		strcpy(config_outdir, iniSettings.out_dir);
	}
	else {
		set_out_dir("");
		strcpy(config_outdir, "");
	}

	for (frame = 0; frame < 7; frame++)
		hbm[frame] = LoadImage(hinst, MAKEINTRESOURCE(IDB_BITMAP9 + frame), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	frame = 0;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	for (frame = 0; frame < 7; frame++) 
		DeleteObject(hbm[frame]);

	return msg.wParam;
}

void HandleDrag(HWND hwnd, HDROP hDrop)
{
	int    cFiles,
	       i,
	       flag = 0;
	char   szFile[MAX_PATH],
	       *ext;

	cFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
	for (i = 0; i < cFiles; i++) {
		DragQueryFile(hDrop, i, szFile, sizeof(szFile));

		if (ext = strrchr(szFile, '.'))	{
			if (stricmp(ext, ".wav") == 0)	{
				flag = 1;
				encthread_addfile(szFile);
				encoding = 1;
			}
			else
				MessageBox(NULL, "Unsupported input file type", NULL, MB_OK | MB_ICONEXCLAMATION);
		}
	}

	DragFinish(hDrop);

	if (flag)
		SetEvent(event);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC          hdc,
	             hmem;
	static HDC   offscreen;
	PAINTSTRUCT  ps;
	RECT         rect;
	BITMAP       bm;
	POINT        point;
	static POINT start;
	static int   dragging = 0;
	HDC          desktop;
	HBITMAP      hbitmap;
	HANDLE       hdrop;
	HFONT        dfltFont;
	int          dfltBGMode;
	double       percomp;

	switch (message) {
		case WM_CREATE:

			menu = LoadMenu(hinst, MAKEINTRESOURCE(IDR_MENU1));
			menu = GetSubMenu(menu, 0);

			offscreen = CreateCompatibleDC(NULL);
			desktop = GetDC(GetDesktopWindow());
			hbitmap = CreateCompatibleBitmap(desktop, 200, 200);
			ReleaseDC(GetDesktopWindow(), desktop);
			SelectObject(offscreen, hbitmap);

			// Start the engines
			encthread_init();

			// We accept drag&drop
			DragAcceptFiles(hwnd, TRUE);
			return 0;

		case WM_PAINT:

			hdc = BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rect);
			width = rect.right + 1;
			height = rect.bottom + 1;

			FillRect(offscreen, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));
			DrawText(offscreen, "Drop Files Here", -1, &rect, DT_SINGLELINE | DT_CENTER);

			hmem = CreateCompatibleDC(offscreen);
			SelectObject(hmem, hbm[frame]);
			GetObject(hbm[frame], sizeof(BITMAP), &bm);
			BitBlt(offscreen, width / 2 - 33, height / 2 - 33, bm.bmWidth, bm.bmHeight, hmem, 0, 0, SRCCOPY);
			DeleteDC(hmem);
		
			percomp = ((double)(totalfiles - numfiles) + 1 - (1 - file_complete)) / (double)totalfiles;

			SetRect(&vbrBR, 0, height - 35, width, height - 19);

			dfltBGMode = SetBkMode(offscreen, TRANSPARENT);
			dfltFont = SelectObject(offscreen, font2);

			SetRect(&bar1, 0, height - 23, (int)(file_complete * width), height - 13);
			SetRect(&bar2, 0, height - 12, (int)(percomp * width), height - 2);

			FillRect(offscreen, &bar1, (HBRUSH)GetStockObject(LTGRAY_BRUSH));
			FillRect(offscreen, &bar2, (HBRUSH)GetStockObject(DKGRAY_BRUSH));

			if (fileName[0]) {
				char  fileCaption[80];

				strcpy(fileCaption, " ");
				strncat(fileCaption, fileName, 78);

				DrawText(offscreen, fileCaption, -1, &bar1, DT_SINGLELINE | DT_LEFT);
			}

			SelectObject(offscreen, dfltFont);
			SetBkMode(offscreen, dfltBGMode);

			BitBlt(hdc, 0, 0, width, height, offscreen, 0, 0, SRCCOPY);

			EndPaint(hwnd, &ps);

			DefWindowProc(hwnd, message, wParam, lParam);
			return 0;

		case WM_TIMER:

			if (animate || frame) {
				frame++;
				if (frame > 6) 
					frame = 0;
			}
			else {
				frame = 0;
			}
			GetClientRect(hwnd, &rect);
			InvalidateRect(hwnd, &rect, FALSE);
			return 0;

		case WM_LBUTTONDOWN:

			start.x = LOWORD(lParam);
			start.y = HIWORD(lParam);
			ClientToScreen(hwnd, &start);
			GetWindowRect(hwnd, &rect);
			start.x -= rect.left;
			start.y -= rect.top;
			dragging = 1;
			SetCapture(hwnd);
			return 0;

		case WM_LBUTTONUP:

			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_MOUSEMOVE:

			if (dragging) {
				point.x = LOSHORT(lParam);
				point.y = HISHORT(lParam);

				/* lParam can contain negative coordinates !
				point.x = LOWORD(lParam);
				point.y = HIWORD(lParam);
				*/

				ClientToScreen(hwnd, &point);
				SetWindowPos(hwnd, 0, point.x - start.x, point.y - start.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW);
				iniSettings.window_x = point.x - start.x;
				iniSettings.window_y = point.y - start.y;
			}
			return 0;

		case WM_CAPTURECHANGED:

			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_RBUTTONUP:

			point.x = LOWORD(lParam);
			point.y = HIWORD(lParam);
			ClientToScreen(hwnd, &point);
			TrackPopupMenu(menu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
			return 0;

		case WM_COMMAND:

			switch (LOWORD(wParam)) {
 
				case IDM_QUIT: {

					char PathAndName[MAX_PATH];
					strcpy(PathAndName, CmdDir);
					strcat(PathAndName, INI_FILE);
					WriteIniFile(PathAndName);
					encoding_done = 1;
					PostQuitMessage(0);
				}
					break;

				case IDM_ONTOP:

					set_always_on_top(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;	

				case IDM_LOGERR:

					set_logerr(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;

				case IDM_DELINPUT:

					set_delInput(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;

				case IDM_SAVEQUALITY: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_QUALITY), hwnd, ResampleProc);

					break;
				}
				case IDM_SAVEDIR: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_DIRECTORY), hwnd, DirProc);

					break;
				}
				case IDM_ABOUT: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutProc);

					break;
				}
			} // LOWORD(wParam)
			return 0;

		case WM_DROPFILES:

			hdrop = (HANDLE)wParam;
			HandleDrag(hwnd, hdrop);
			return 0;
		

		case WM_DESTROY:

			encoding_done = 1;
			PostQuitMessage(0);
			return 0;

		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
	}
	return 0;
}

/*
 *  Resample parameters dialog procedures.
 */

BOOL CALLBACK ResampleProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	int      br;

	switch (message) {
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			for (br=0; resampleCaption[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) resampleCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(resampleCaption[br]));
			}

			/* set to saved sample rate */
			for (br=0; resampleCaption[br] != 0; br++) {
				if ( atoi(resampleCaption[br]) == iniSettings.resample_rate) {
					SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETCURSEL,
	        	  			(WPARAM) br, (LPARAM) 0);
					break;
				}
			}

			for (br=0; converter[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLER, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) converter[br]);

				SendDlgItemMessage(hwndDlg, IDC_RESAMPLER, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) converter[br]);
			}

			/* set to saved sample rate converter*/
			for (br=0; converter[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLER, CB_SETCURSEL,
        	  			(WPARAM) iniSettings.converter, (LPARAM) 0);
				break;
			}

			for (br=0; wavFormat[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_OUTBITWIDTH, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) wavFormat[br]);

				SendDlgItemMessage(hwndDlg, IDC_OUTBITWIDTH, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) wavFormat[br]);
			}

			SendDlgItemMessage(hwndDlg, IDC_OUTBITWIDTH, CB_SETCURSEL,
	            		(WPARAM) iniSettings.wav_format, (LPARAM) 0);

			SendDlgItemMessage(hwndDlg, IDC_FILE_SUFFIX, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.file_suffix);

			return TRUE;
		}
		case WM_CLOSE:
			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON1: {
					char szTemp[64];

					if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
								CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0));
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLER, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_converter(br);
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_OUTBITWIDTH, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_wav_format(br);
					}

					GetDlgItemText(hwndDlg, IDC_FILE_SUFFIX, szTemp, sizeof(szTemp));
					if (strlen(szTemp) > 0) {
						set_file_suffix(szTemp);
					}
					else {
						set_file_suffix("");
					}
					EndDialog(hwndDlg, FALSE);
				}
				break;
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;
        	  	}
	}
	return FALSE; 
}

/*
 *  Directory parameters dialog procedures.
 */

BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{

	switch (message) { 
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if(iniSettings.out_dir_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_USEOTHERDEF,TRUE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, config_outdir);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_USESAMEDIR,TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
			}
			return TRUE;
		}

		case WM_CLOSE:
			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON4: {
					char     szTemp_2[MAX_PATH];
					int      i = 0;
					int      j = 0;
					char     sub[MAX_PATH];

					if (IsDlgButtonChecked(hwndDlg, IDC_USESAMEDIR) == BST_CHECKED) {
						strcpy(config_outdir, "");
						set_out_dir("");
						set_out_dir_mode(0);
					}
					else {
						GetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp_2, sizeof(szTemp_2));
						if(!szTemp_2[0]) {
							set_out_dir_mode(0);
							strcpy(config_outdir, "");
						}
						else {
							j = strlen(szTemp_2) - 1;
							if(szTemp_2[j] != '\\') {
								szTemp_2[j+1] = '\\';
								szTemp_2[j+2] = '\0';
							}
							for(i = 0; i < strlen(szTemp_2); i++) {
								if(szTemp_2[i] == '\\') {
									sub[i] = szTemp_2[i];
									sub[i+1] = '\0';
									CreateDirectory(sub, NULL);
								}
								else
									sub[i] = szTemp_2[i];
							}
							sub[i-1] = '\0';
							strcpy(config_outdir, sub);
						}
						set_out_dir_mode(IsDlgButtonChecked(hwndDlg, IDC_USEOTHERDEF) == BST_CHECKED ? 1 : 0);
						set_out_dir(config_outdir);
					}

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_USESAMEDIR:
					SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
					break;
									
				case IDC_USEOTHERDIR:
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

				case IDC_USEOTHERDEF:
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

				case IDC_SETDIR: {
					char        szTemp[MAX_PATH];
					char        name[MAX_PATH];
					BROWSEINFO  bi;
					ITEMIDLIST  *idlist;

					bi.hwndOwner = hwndDlg;
					bi.pidlRoot = 0;
					bi.pszDisplayName = name;
					bi.lpszTitle = "Select a directory for output:";
					bi.ulFlags = BIF_RETURNONLYFSDIRS;
					bi.lpfn = BrowseCallbackProc;
					bi.lParam = 0;
					idlist = SHBrowseForFolder( &bi );

					if (idlist) 
						SHGetPathFromIDList( idlist, szTemp );
					else
						strcpy(szTemp,"");

					SetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp);
				}
				break;

				default:
					break;
        	  	}
	}
	return FALSE; 
}

/*
 *  About dialog procedures.
 */
BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	switch (message) { 
		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);
		}
		return TRUE;

		case WM_CLOSE:
			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDCANCEL:
					EndDialog(hwndDlg, TRUE);
					return TRUE;

        	  	}
	}
	return FALSE; 
}


/********************************** end of main.c **********************************/



