/* SRCDrop - Filename: RESAMPLE.C
 *
 *   Copyright (c) 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "misc.h"

#include "audio.h"
#include "resample.h"
#include "..\src\samplerate.h"

int src_proc(src_proc_opt *src_opts)
{
	float 	*input = malloc(sizeof(float) * READSIZE * src_opts->channels);
	float 	*output = malloc(sizeof(float) * READSIZE * src_opts->channels);

	output_audio_file *aufile;
	SRC_STATE	  *src_state;
	SRC_DATA	  src_data;
	int		  error;
	double		  max = 0.0;
	int		  output_count = 0;
	unsigned long     samplesdone = 0;

	aufile = open_output_audio_file(src_opts);

	if (aufile == NULL) {
		error_handler("ERROR: Cannot open output file %s", src_opts->filename);
		return (0) ;
	}

	/* Initialize the sample rate converter.
	 */
	if ((src_state = src_new (src_opts->converter, src_opts->channels, &error)) == NULL)
	{
		error_handler("Error : src_new() failed : %s.", src_strerror (error)) ;
		return (0) ;
		} ;

	src_data.end_of_input = 0 ; /* Set this later. */

	/* Start with zero to force load in while loop.
	 */
	src_data.input_frames = 0;
	src_data.data_in = input;

	src_data.src_ratio = (1.0 * src_opts->resamplefreq) / src_opts->rate;

	src_data.data_out = output;
	src_data.output_frames = READSIZE;

	while (1)
	{
		/* If the input buffer is empty, refill it.
		 */
		if (src_data.input_frames == 0) {
			src_data.input_frames = src_opts->read_samples(src_opts->readdata, input, READSIZE);
			src_data.data_in = input;

			samplesdone += src_data.input_frames;

			/* Call progress update every 10 pages
			 */
			if (samplesdone % 100 == 0) {
				src_opts->progress_update(src_opts->total_samples_per_channel, samplesdone);
			}

			/* The last read will not be a full buffer, so end_of_input.
			 */
			if (src_data.input_frames < READSIZE)
				src_data.end_of_input = 1;
		}

		if ((error = src_process (src_state, &src_data))) {
			error_handler("nError : %s\n", src_strerror (error));
			return (0) ;
		}

		/* Terminate if done.
		 */
		if (src_data.end_of_input && src_data.output_frames_gen == 0)
			break;

		/* Write output.
		 */
		{
			void *sample_buffer;
			sample_buffer = malloc(sizeof(float) * src_opts->channels * src_data.output_frames_gen);

			sample_buffer = output_to_PCM(output, sample_buffer, src_opts->channels,
					src_data.output_frames_gen, src_opts->format);

			write_audio_file(aufile, sample_buffer, src_data.output_frames_gen * src_opts->channels);

			free(sample_buffer);

			output_count += src_data.output_frames_gen;

			src_data.data_in += src_data.input_frames_used * src_opts->channels;
			src_data.input_frames -= src_data.input_frames_used;
		}
	}
	src_state = src_delete(src_state);
	free(input);
	free(output);
	close_audio_file(aufile, src_opts);

	return output_count;
} /* End of resample.c */

