/* SRCDrop - Filename: RESAMPLE.H
 *
 *   Copyright (c) 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __RESAMPLE_H
#define __RESAMPLE_H

#include <stdio.h>
#include <windows.h>

#define READSIZE 2048

typedef void TIMER;
typedef long (*audio_read_func)(void *src,
                                float *buffer,
                                int samples);

typedef void (*progress_func)(long totalsamples,
                              long samples);

typedef void (*error_func)(char *errormessage);

int create_directories(char *);

void encode_error(char *errmsg);

typedef struct
{
	char *outfile;
	int resamplefreq;
	int converter;
	int output_format;
	char *file_suffix;
	int logerr;
	int delete_input;
} src_options;

typedef struct
{
	audio_read_func read_samples;
	progress_func progress_update;
	error_func error;
	
	void *readdata;

	long total_samples_per_channel;
	int channels;
	int channel_mask;
	long rate;
	int samplesize;
	int resamplefreq;
	int converter;
	int format;
	int logerr;

	char *filename;
} src_proc_opt;

int src_proc(src_proc_opt *src_opts);

typedef struct {
	int	window_x;
	int	window_y;
	int	always_on_top;
	int	logerr;
	int	delete_input;
	int     resample_rate;
	int     converter;
	int     wav_format;
	char    file_suffix[64];
	int     out_dir_mode;
	char    out_dir[MAX_PATH];
} SettingsSRC;

/* GLOBALS */
extern SettingsSRC iniSettings;

#endif /* __RESAMPLE_H */
